Type: Package
Package: metatools
Title: Enable the Use of 'metacore' to Help Create and Check Dataset
Version: 0.3.0
Authors@R: c(
    person("Liam", "Hobby", , "liam.f.hobby@gsk.com", role = c("aut", "cre")),
    person("Christina", "Fillmore", , "christina.e.fillmore@gsk.com", role = "aut",
           comment = c(ORCID = "0000-0003-0595-2302")),
    person("Bill", "Denney", role = "aut"),
    person("Mike", "Stackhouse", , "mike.stackhouse@atorusresearch.com", role = "aut",
           comment = c(ORCID = "0000-0001-6030-723X")),
    person("Jana", "Stoilova", , "jana.stoilova@roche.com", role = "aut"),
    person("Tamara", "Senior", , "tamara.senior@roche.com", role = "aut"),
    person("GlaxoSmithKline LLC", role = c("cph", "fnd")),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd")),
    person("Atorus Research LLC", role = c("cph", "fnd"))
  )
Description: Uses the metadata information stored in 'metacore' objects to
    check and build metadata associated columns.
License: MIT + file LICENSE
URL: https://github.com/pharmaverse/metatools,
        https://pharmaverse.github.io/metatools/
BugReports: https://github.com/pharmaverse/metatools/issues
Depends: R (>= 4.1.0)
Imports: cli, dplyr, lifecycle, magrittr, metacore (>= 0.2.0), purrr,
        rlang, stringr, tibble, tidyr
Suggests: covr, haven, pharmaversesdtm, safetyData, spelling, testthat
        (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-28 11:12:23 UTC; lfh66519
Author: Liam Hobby [aut, cre],
  Christina Fillmore [aut] (ORCID:
    <https://orcid.org/0000-0003-0595-2302>),
  Bill Denney [aut],
  Mike Stackhouse [aut] (ORCID: <https://orcid.org/0000-0001-6030-723X>),
  Jana Stoilova [aut],
  Tamara Senior [aut],
  GlaxoSmithKline LLC [cph, fnd],
  F. Hoffmann-La Roche AG [cph, fnd],
  Atorus Research LLC [cph, fnd]
Maintainer: Liam Hobby <liam.f.hobby@gsk.com>
Repository: CRAN
Date/Publication: 2026-04-28 11:30:02 UTC
Built: R 4.6.0; ; 2026-04-28 15:48:27 UTC; unix
