# Generated by roxygen2: do not edit by hand

export("%>%")
export(add_labels)
export(add_variables)
export(build_from_derived)
export(build_qnam)
export(check_ct_col)
export(check_ct_data)
export(check_unique_keys)
export(check_variables)
export(combine_supp)
export(convert_var_to_fct)
export(create_cat_var)
export(create_subgrps)
export(create_var_from_codelist)
export(drop_unspec_vars)
export(get_bad_ct)
export(make_supp_qual)
export(metatools_example)
export(order_cols)
export(remove_labels)
export(set_variable_labels)
export(sort_by_key)
importFrom(cli,ansi_collapse)
importFrom(cli,cli_abort)
importFrom(cli,cli_inform)
importFrom(cli,cli_text)
importFrom(cli,cli_warn)
importFrom(cli,col_red)
importFrom(cli,qty)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,enquo)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_split)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pick)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(magrittr,"%>%")
importFrom(metacore,get_control_term)
importFrom(metacore,get_keys)
importFrom(metacore,select_dataset)
importFrom(metacore,verify_DatasetMeta)
importFrom(purrr,discard)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(purrr,map_lgl)
importFrom(purrr,pmap_dfr)
importFrom(purrr,reduce)
importFrom(purrr,safely)
importFrom(purrr,walk2)
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,as_label)
importFrom(rlang,as_name)
importFrom(rlang,enexpr)
importFrom(rlang,expr)
importFrom(rlang,is_logical)
importFrom(rlang,is_named)
importFrom(rlang,list2)
importFrom(rlang,set_names)
importFrom(rlang,sym)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_match_all)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_split)
importFrom(stringr,str_to_lower)
importFrom(stringr,str_to_upper)
importFrom(stringr,str_trim)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
importFrom(utils,capture.output)
