globalVariables(c(
  ".",
  ".data",
  "code",
  "decode",
  "DOMAIN",
  "IDVAR",
  "IDVARVAL",
  "QEVAL",
  "QLABEL",
  "QNAM",
  "QORIG",
  "QVAL",
  "RDOMAIN",
  "STUDYID",
  "USUBJID",
  "code_id",
  "col_name",
  "derivation",
  "derivation_id",
  "ds",
  "idvar",
  "key_seq",
  "label",
  "origin",
  "qeval",
  "supp_flag",
  "type",
  "variable",
  "merge_on"
))


#' Pipe operator
#'
#' See \code{magrittr::\link[magrittr:pipe]{\%>\%}} for details.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
#' @param lhs A value or the magrittr placeholder.
#' @param rhs A function call using the magrittr semantics.
#' @return The result of calling `rhs(lhs)`.
NULL
